aboutsummaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
authorAmit Kapila <akapila@postgresql.org>2019-02-02 15:47:00 +0530
committerAmit Kapila <akapila@postgresql.org>2019-02-02 15:47:00 +0530
commit0b8bdb3c3ec092f18813d5b6fc738d7ec450289b (patch)
tree5a02375454059ffe82c48659dfe9b71e90b14ae6 /src
parent3e938a83b2ee3b47d47cdbdf58df09946877d4fd (diff)
downloadpostgresql-0b8bdb3c3ec092f18813d5b6fc738d7ec450289b.tar.gz
postgresql-0b8bdb3c3ec092f18813d5b6fc738d7ec450289b.zip
Avoid possible deadlock while locking multiple heap pages.
To avoid deadlock, backend acquires a lock on heap pages in block number order. In certain cases, lock on heap pages is dropped and reacquired. In this case, the locks are dropped for reading in corresponding VM page/s. The issue is we re-acquire locks in bufferId order whereas the intention was to acquire in blockid order. This commit ensures that we will always acquire locks on heap pages in blockid order. Reported-by: Nishant Fnu Author: Nishant Fnu Reviewed-by: Amit Kapila and Robert Haas Backpatch-through: 9.4 Discussion: https://postgr.es/m/5883C831-2ED1-47C8-BFAC-2D5BAE5A8CAE@amazon.com
Diffstat (limited to 'src')
-rw-r--r--src/backend/access/heap/hio.c8
1 files changed, 4 insertions, 4 deletions
diff --git a/src/backend/access/heap/hio.c b/src/backend/access/heap/hio.c
index 3da0b49ccc4..5839c168e6c 100644
--- a/src/backend/access/heap/hio.c
+++ b/src/backend/access/heap/hio.c
@@ -115,8 +115,8 @@ ReadBufferBI(Relation relation, BlockNumber targetBlock,
* visibility map page, if we haven't already got one.
*
* buffer2 may be InvalidBuffer, if only one buffer is involved. buffer1
* must not be InvalidBuffer. If both buffers are specified, buffer1 must
* be less than buffer2.
*/
static void
GetVisibilityMapPins(Relation relation, Buffer buffer1, Buffer buffer2,
@@ -127,7 +127,7 @@ GetVisibilityMapPins(Relation relation, Buffer buffer1, Buffer buffer2,
bool need_to_pin_buffer2;
Assert(BufferIsValid(buffer1));
- Assert(buffer2 == InvalidBuffer || buffer1 <= buffer2);
+ Assert(buffer2 == InvalidBuffer || block1 <= block2);
while (1)
{
@@ -465,7 +465,7 @@ loop:
* done a bit of extra work for no gain, but there's no real harm
* done.
*/
- if (otherBuffer == InvalidBuffer || buffer <= otherBuffer)
+ if (otherBuffer == InvalidBuffer || targetBlock <= otherBlock)
GetVisibilityMapPins(relation, buffer, otherBuffer,
targetBlock, otherBlock, vmbuffer,
vmbuffer_other);
@@ -115,8 +115,8 @@ ReadBufferBI(Relation relation, BlockNumber targetBlock,
*
* buffer2 may be InvalidBuffer, if only one buffer is involved. buffer1
- * must not be InvalidBuffer. If both buffers are specified, buffer1 must
- * be less than buffer2.
+ * must not be InvalidBuffer. If both buffers are specified, block1 must
+ * be less than block2.
*/
static void